/**
 * Copyright (c) 2006 Sascha Grau
 * contact: rostar@graui.de
 */

#include "graph.h"

Node::Node(unsigned long id, char* c_name, unsigned long degree) {
    this->id = id;
    deg_bound = degree;
    if (c_name != NULL)
        name = new std::string(c_name);
    else
        name = NULL;
}

Node::~Node() {
    delete name;
    if (!edges.empty())
        edges.clear();
}

bool Node::full() {
    // if there is a degree bound, may new edges be added ?
    return (deg_bound > 0) && (edges.length() >= deg_bound);
}

bool Node::addEdge(Edge* e) {

    if (full()) return false;

    edges.push_back(e);
    return true;
}

Edge::Edge() {
    a = NULL;
    b = NULL;
    cost = 0;
    label = NULL;
    forest = 0;
}

Edge::Edge(Node* a, Node* b, unsigned long cost) {
    this->a = a;
    this->b = b;
    this->cost = cost;
    label = NULL;
    forest = 0;
}

/** Edge cost and edge nodes must match,
 *  regardless of their order (undirected).
 */
bool Edge::operator==(const Edge& e) const {

    return (this->cost == e.cost)
            && (((a->id == e.a->id)
                  && (b->id == e.b->id))
                ||
                ((a->id == e.b->id) 
                  && (b->id == e.a->id)));
}

/**
 * Return adjacent Node, or NULL if 'from'
 * is not an incident Node.
 */
Node* Edge::traverse(const Node* from) const {

    if      (from == a) return b;
    else if (from == b) return a;
    else return NULL;
}

Graph::~Graph() {

    while (!edges.empty()) {
        Edge* e = edges.pop_front();
        delete e;
    }

    while (!nodes.empty()) {
        Node* v = nodes.pop_front();
        delete v;
    }
}

