/**
 * Copyright (c) 2006 Sascha Grau
 * contact: rostar@graui.de
 */

#ifndef GRAPH_H_
#define GRAPH_H_

#include <vector>
#include <map>
#include "dllist.h"

struct Edge;

struct Node {

    Node(unsigned long id = 0, char* c_name = NULL, unsigned long degree = 0);
    ~Node();

    unsigned long id;
    DLList<Edge*> edges;
    std::map<unsigned int, Edge*> upwards;
    unsigned int deg_bound;
    std::string* name;

    bool addEdge(Edge* e);
    bool full();
    bool operator<(const Node& n) const { return this->id < n.id; }
};

struct Edge {

    Edge();
    Edge(Node* a, Node* b, unsigned long cost);

    Node* a;
    Node* b;
    unsigned long cost;

    Edge* label;
    unsigned int forest;

    bool operator==(const Edge& e) const;
    bool operator<(const Edge& e) const { return this->cost < e.cost; }
    Node* traverse(const Node* from) const;
};

class Graph {

public:
    Graph() {}
    ~Graph();

    DLList<Edge*> edges;
    DLList<Node*> nodes;

    unsigned long edgeCount() { return edges.length(); }
    unsigned long nodeCount() { return nodes.length(); }
};
#endif /* GRAPH_H_ */
