/**
 * Copyright (c) 2006 Sascha Grau
 * contact: rostar@graui.de
 */

#include "graphexport.h"

const std::string GraphExport::DOT_HEADER =
    std::string("strict graph Result {\n") +
    std::string("graph[ratio=fill,center=1]\n") +
    std::string("node [shape = circle, style=solid]\n") +
    std::string("edge [style = \"setlinewidth(3)\"];\n");

const std::string GraphExport::colors[] =
    { "black", "red", "green", "blue", "darkorange1",
      "cyan3", "bisque3", "gold", "brown", "blueviolet",
      "palegreen4", "yellow", "violetred2", "navy",
    };

using namespace std;

void GraphExport::toDotStream(ostream& out, Graph* g, unsigned long forest) {

    out << DOT_HEADER;

    /** cool features of tomorrow:
     * export spanning trees as subgraphs */

    DLListIterator<Edge*>* it = g->edges.iterator();
    int c = 0;
    while(!it->endReached()) {
        Edge* e = it->getElem();
        if ((forest == ALL) || (e->forest == forest)) {
            /* nodes */
            out << '\"';
            if (e->a->name)  out << *e->a->name;  else out << e->a->id;
            out << "\" -- \"";
            if (e->b->name)  out << *e->b->name;  else out << e->b->id;
            /* cost */
            out << "\" [ label=\"" << e->cost << '\"';
            /* forest */
            c = (e->forest >= colornum) ? 0 : e->forest;
            out << " color=\"" << colors[c] << "\" ];" <<  std::endl;
        }
        it->next();
    }

    DLListIterator<Node*>* nit = g->nodes.iterator();
    while(!nit->endReached()) {
        Node* n = nit->getElem();
        out << *n->name << std::endl;
        nit->next();
    }

    out << "}" << std::endl;
}

void GraphExport::toDotFile(string path, Graph* g, unsigned long forest)
                          throw(GraphIOException) {

    ofstream fout(path.c_str(), ios_base::out | ios_base::trunc);

    if (!fout)
        throw (GraphIOException("Could not open file " + path + " for writing."));

    toDotStream(fout, g, forest);
    fout.close();
}

void GraphExport::printGraph(Graph* g) {

    DLListIterator<Edge*>* it = g->edges.iterator();
    std::cout << "*** Graph *** " << g->edges.length() << " egdes." << std::endl;
    while(!it->endReached()) {
        Edge* e = it->getElem();
        if (e->a->name)  std::cout << *e->a->name;  else std::cout << e->a->id;
        std::cout << " -- ";
        if (e->b->name)  std::cout << *e->b->name;  else std::cout << e->b->id;
        std::cout << " cost " <<  e->cost << " forest " << e->forest << std::endl;
        it->next();
    }
    delete it;
}
