/**
 * Copyright (c) 2006 Sascha Grau
 * contact: rostar@graui.de
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "vn/vn_graph.h"

#include "graphviz/agraph.h"

void printusage(char* myname) {
    printf("usage:\n");
    printf("%s <n> <p> <outfile>\n", myname);
    printf("<n> - node number\n");
    printf("<p> - edge probability (decimal delimiter '.')\n");
    exit(-1);
}

int main(int argc, char** argv) {

    /** cmd-line */
    if (argc != 4) {
        printusage(argv[0]);
    }

    errno = 0;
    int tmp = atoi(argv[1]);
    if ((errno != 0) || (tmp <= 0))
        printusage(argv[0]);

    unsigned int n = (unsigned int) tmp;

    errno = 0;
    double p = strtod(argv[2], (char **)NULL);
    if ((errno != 0) || (p <= 0) || (p > 1))
        printusage(argv[0]);

    /** generate graph */
    graph_t g = graph_new(n);
    graph_gnp(g, p);
    graph_make_dotfile(g, argv[3]);

    /** reopen it with graphviz and add
     *  weights. */

    Agraph_t* gviz = NULL;

    FILE* f = fopen(argv[3], "r");
    gviz = agread(f, 0);
    fclose(f);

    Agnode_t* v;
    Agedge_t* e;
    unsigned int w = 1;

    for (v = agfstnode(gviz); v; v=agnxtnode(v)) {
        for (e = agfstout(v); e; e = agnxtout(e)) {
            char buffer[30];
            snprintf(buffer, 30, "%d", w++);
            Agsym_t* attr = agattr(gviz, AGEDGE, "label", buffer);
            agxset(e, attr, buffer);
        }
    }

    f = fopen(argv[3], "w");
    fprintf(f, "strict ");
    agwrite(gviz, f);
    fclose(f);

    return 0;
}
