/**
 * Copyright (c) 2006 Sascha Grau
 * contact: rostar@graui.de
 */

#include "../ds/dllist.h"
#include "../ds/queue.h"
#include "../ds/multipart.h"

#include <stdlib.h>
#include <iostream>

const unsigned int elements = 20;
const unsigned int partitions = 2;

using namespace std;

int main() {

/***Multipart******************************************/

    Multipart* p = new Multipart(elements, partitions);

    for (unsigned int i=0; i<elements; i++) {

        if (p->find(i) != i) {
            cout << "Predecessor initialization failed." << endl;
            return -1;
        }

        if (i > 0)
            p->unite(i, i-1);

        if (p->find(i) != p->find(0)) {
            cout << "union operator failed: union(";
            cout << i-1 << "," << i << ")" << endl;
            p->dbg_print_part();
            return -1;
        }

        if ((i > 2) && (p->find(0) == 0)) {
            cout << "union balancing failed" << endl;
            p->dbg_print_part();
            return -1;
        }
    }

/***DLList::sort()*************************************/

    DLList<long> list;
    long* val = new long[elements];
    DLList<long*> list2;

    for (unsigned int i=0; i<elements; i++) {
        long rand = random();
        list.push_back(rand);
        if (list.back() != rand)  {
            cout << "list.back() disfunctional." << endl;
            return -1;
        }
        val[i] = rand;
        list2.push_back(&(val[i]));
    }

    list.sort();
    list2.sort();

    DLListIterator<long>* it = list.iterator();
    long last = 0;
    long cur  = 0;

    while (it->hasMore()) {
        last = cur;
        cur  = it->next();
        if ((last != 0) && (last > cur)) {
            cout << "double linked list standard sorting failed." << endl;
            cout << "length: " << list.length() << endl;
            cout << "elements: " << last << ":" << cur << endl;
            list.print();
        }
    }

    delete it;

    DLListIterator<long*>* it2 = list2.iterator();
    long* last2 = NULL;
    long* cur2  = NULL;

    while (it2->hasMore()) {
        last2 = cur2;
        cur2  = it2->next();
        if ((last2 != NULL) && (*last2 > *cur2)) {
            cout << "double linked list pointer sorting failed." << endl;
        }
    }
    delete it2;

    Queue<long> q1;
    Queue<long*> q2;

    it = list.iterator();
    while (!it->endReached()) {
        q1.enq(it->next());
    }
    delete it;

    it2 = list2.iterator();
    while (!it2->endReached()) {
        q2.enq(it2->next());
    }
    delete it2;

    for (unsigned int i=0; i<elements; i++) {
        q1.deq();
        q2.deq();
    }

    if (!q1.empty()) {
        cout << "std queue did not run empty" << endl;
        q1.print();
    }

    if (!q2.empty()) {
        cout << "ptr queue did not run empty" << endl;
        q2.print();
    }

    return 0;
}
